#lang racket

;; -- implementation of the "middle school" algorithm
;; -- for computing GCD, using their common factors

(define gcd
  (lambda (m n)
    (apply * (common-factors (factor m) (factor n)))))

; > (gcd 60 24)
; 12
; > (gcd 60 29)
; 1

;; -- a procedure to factor any integer >= 2

(define factor
  (lambda (n)
    (letrec ((factor-aux (lambda (n k factors)
                         (cond ((= n k)
                                  (reverse (cons n factors)))
                               ((divides? n k)
                                  (factor-aux (/ n k) k (cons k factors)))
                               (else
                                  (factor-aux n (+ k 1) factors))))))
      (factor-aux n 2 '()))))

(define divides?
  (lambda (m n)
    (zero? (remainder m n))))

; > (factor 60)
; '(2 2 3 5)
; > (factor 61)
; '(61)
; > (factor 62)
; '(2 31)

;; -- a procedure to compute the common values
;; -- in two sorted lists of values

(define common-factors
  (lambda (lst1 lst2)
    (if (or (null? lst1) (null? lst2))
        '()
        (if (= (car lst1) (car lst2))
            (cons (car lst1) (common-factors (cdr lst1) (cdr lst2)))
            (if (< (car lst1) (car lst2))
                (common-factors (cdr lst1) lst2)
                (common-factors lst1 (cdr lst2)))))))

; > (common-factors '(2 5 7) '(2 2 2 2 2))
; (2)
; > (common-factors '(2 2 5 7) '(2 3 3 5 5 7 7 11))
; (2 5 7)
